var Etcd = require('./promise.etcd');
var Q = require('q');
var _ = require('underscore');
var HashRing = require('hashring');

function Frodo(config) {
  this.config = config || {};
  this.context = this.config.context || 'frodo';
  this.ready = Q();
  var rawServers = this.config.etcdServers || [
    { host: '127.0.0.1', port: 4001 }
  ];
  if (!_.isArray(rawServers)) {
    rawServers = [ rawServers ];
  }
  var servers = [];
  rawServers.forEach(function (server) {
    if (_.isString(server)) {
      servers.push({ host: server, port: 4001 });
    } else if (_.isObject(server)) {
      servers.push({ host: server.host || '127.0.0.1', port: server.port || 4001 });
    } else {
      throw new Error(server + ' as a server should be an address <string> or { host: <string>, port: <integer> }')
    }
  });

  // TODO: Support more than one server
  this.etcd = new Etcd(servers[0].host, servers[0].port, servers[0].sslopts);

  if (!this.config.lazy) {
    this.ready = this.init();
  }
}

Frodo.prototype.init = function () {
  var self = this;
  self.hosts = [];
  self.hashRing = new HashRing(self.hosts, 'md5');
  self.hostsWatcher = self.etcd.watcher(self.context + '/hosts', null, { recursive: true });
  self.hostsWatcher.on('change', function (result) {
    if (result.action == 'set' && result.node.value) {
      self.hosts.push(result.node.value);
      self.hashRing.add(result.node.value)
    } else if (result.action == 'delete' && result.prevNode.value) {
      var indexOf = self.hosts.indexOf(result.prevNode.value);
      if (indexOf >= 0) {
        self.hosts.splice(indexOf, 1);
        self.hashRing.remove(result.prevNode.value);
      }
    }
  });

  return self.etcd.get(self.context + '/hosts', { recursive: true }).then(function (result) {
    if (result && result[0]) {
      self.hosts = _.pluck(result[0].node.nodes, 'value');
      self.hashRing.add(self.hosts);
    }
  }, function (err) {
    if (err.errorCode == 100) {
      return self.etcd.mkdir(self.context + '/hosts/');
    } else {
      return Q.reject(err);
    }
  });
};

Frodo.prototype.addServer = function (server) {
  var self = this;
  return this.ready.then(function () {
    return self.etcd.set(self.context + '/hosts/' + server, server);
  });
};

Frodo.prototype.removeServer = function (server) {
  var self = this;
  return this.ready.then(function () {
    return self.etcd.del(self.context + '/hosts/' + server);
  });
};

Frodo.prototype.servers = function () {
  var self = this;
  return this.ready.then(function () {
    return self.hosts;
  });
};

Frodo.prototype.serverById = function (id) {
  var self = this;
  return this.ready.then(function () {
    return self.hashRing.get(id);
  });
};

Frodo.prototype.close = function () {
  this.hostsWatcher.stop();
  return Q();
};

module.exports = Frodo;