var expect = require('chai').expect;
var sinon = require('sinon');
var Etcd = require('node-etcd');
var Frodo = require('../../src/index');
var Q = require('q');

describe('Frodo integration tests', function () {
  var frodo;

  beforeEach(function (done) {
    frodo = new Frodo({ context: 'integration', lazy: true });
    frodo.etcd.rmdir('integration', { recursive: true }).fin(function () {
      frodo.init().then(function () {
        done();
      }, function (err) {
        console.log(JSON.stringify(err));
        done(err);
      });
    });
  });

  afterEach(function (done) {
    frodo.close().then(done);
  });


  describe('#servers, #addServer', function () {
    it('Should get all servers', function (done) {
      var watcher = frodo.etcd.watcher('integration/hosts/done');
      frodo.addServer('first').then(function () {
        frodo.addServer('second').then(function () {
          frodo.addServer('done');
        });
      });
      watcher.on('change', function (changed) {
        frodo.servers().then(function (servers) {
          expect(servers[0]).to.eql('first');
          expect(servers[1]).to.eql('second');
          watcher.stop();
          done();
        }).done();
      });
    });
  });

  describe('#init', function () {
    it('Should get all servers on init', function (done) {
      var watcher = frodo.etcd.watcher('integration/hosts/done');
      var otherFrodo;
      frodo.addServer('first').then(function () {
        otherFrodo = new Frodo({ context: 'integration' });
        otherFrodo.ready.then(function () {
          return otherFrodo.addServer('done');
        }).done();
      });

      watcher.on('change', function (changed) {
        return frodo.servers().then(function (servers) {
          expect(servers.length).to.lte(2);
          expect(servers[0]).to.equal('first');
          watcher.stop();
          otherFrodo.close();
          done();
        }).done();
      });
    })
  })

  describe('#removeServer', function () {
    it('Should remove servers', function (done) {
      var watcher = frodo.etcd.watcher('integration/hosts/done');
      frodo.addServer('first').then(function () {
        frodo.removeServer('first').then(function () {
          frodo.addServer('second').then(function () {
            frodo.addServer('done');
          });
        })
      });

      watcher.on('change', function (changed) {
        return frodo.servers().then(function (servers) {
          expect(servers[0]).to.equal('second');
          expect(servers[1]).to.not.equal('first');
          watcher.stop();
          done();
        }).done();
      });
    });

    it('Should handle failed on removal of not existing server', function (done) {
      var watcher = frodo.etcd.watcher('integration/hosts/done');
      frodo.removeServer('foo').fail(function () {
        frodo.addServer('done');
      }).done();

      watcher.on('change', function (changed) {
        return frodo.servers().then(function (servers) {
          expect(servers.length).to.lte(1);
          expect(servers[0]).to.not.equal('foo');
          watcher.stop();
          done();
        }).done();
      });
    });

    it ('Should updated when changing hosts from other server', function (done) {
      var watcher = frodo.etcd.watcher('integration/hosts/done');
      var otherFrodo = new Frodo({ context: 'integration' });

      otherFrodo.ready.then(function () {
        return otherFrodo.addServer('first');
      }).then(function () {
        return otherFrodo.addServer('second');
      }).then(function () {
        return otherFrodo.removeServer('first');
      }).then(function () {
        return otherFrodo.addServer('done');
      }).done();

      watcher.on('change', function (changed) {
        return frodo.servers().then(function (servers) {
          expect(servers.length).to.lte(2);
          expect(servers[0]).to.equal('second');
          expect(servers[1]).to.not.equal('first');
          watcher.stop();
          otherFrodo.close();
          done();
        }).done();
      });
    })

  });
});